/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.impl.fingerprint;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import javax.crypto.NoSuchPaddingException;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.integritychecker.fingerprint.Fingerprint;
import lombok.Cleanup;
import lombok.extern.slf4j.Slf4j;
import static java.nio.channels.FileChannel.MapMode.READ_ONLY;

/***********************************************************************************************************************
 * 
 * A facility for computing the fingerprint of a file.
 * 
 * FIXME: no more used. Drop and reuse its test.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=FingerprintComputer.class)
@ThreadSafe @Slf4j
public class FingerprintComputer 
  {
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Nonnull
    public Fingerprint computeFingerprint (final @Nonnull File file, final @Nonnull String algorithm) 
      throws IOException, NoSuchAlgorithmException, NoSuchPaddingException
      {
        log.info("computeFingerprint({}, {})", file, algorithm);
        final @Cleanup RandomAccessFile randomAccessFile = new RandomAccessFile(file, "r");
        final MappedByteBuffer byteBuffer = randomAccessFile.getChannel().map(READ_ONLY, 0, file.length());  
        final MessageDigest digestComputer = MessageDigest.getInstance(algorithm);
        digestComputer.update(byteBuffer);
        randomAccessFile.close();
        return new Fingerprint(algorithm, toString(digestComputer.digest())); 
      }
    
    /*******************************************************************************************************************
     * 
     * 
     * 
     ******************************************************************************************************************/
    @Nonnull
    private static String toString (final @Nonnull byte[] bytes)
      {
        final StringBuilder builder = new StringBuilder();
        
        for (final byte b : bytes)
          {
            final int value = b & 0xff;
            builder.append(Integer.toHexString(value >>> 4)).append(Integer.toHexString(value & 0x0f));
          }
        
        return builder.toString();
      }
  }
